/*
 * Copyright (c) 1989 The Regents of the University of California.
 * All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Mike Muuss.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <dirent.h>
#include <fcntl.h>
#include <sys/mount.h>
#include <sys/statfs.h>
#include <sys/stat.h>
#include <dirent.h>
#include <linux/input.h>
#include <math.h>
#include <poll.h>
#include <getopt.h>
#include<pthread.h>
#include <binder/IPCThreadState.h>
#include <binder/ProcessState.h>
#include <binder/IServiceManager.h>

#include "fingerprint_interface.h"

#define IMG_HEAD_LEN (1078)
#define FEATURE_LEN (7*1024)
#define IMG_BODY_LEN (640*480*4)
#define IMG_TOTAL_LEN (IMG_HEAD_LEN+IMG_BODY_LEN+3*1024)
#define MAX_USER_CNT 100

using namespace android;

static unsigned char fea_buf[FEATURE_LEN]={0,};
static unsigned char fea_buf_match[FEATURE_LEN]={0,};
static unsigned char isoimage[10*1024]={0,};
static unsigned char img_buffer[IMG_TOTAL_LEN]={0,};
static unsigned char bOldInf = 0;
static char nIsoType = 0;

static const char *optstring = "hiv:f:b:m:n:e:VDd:YLg:s:G:S:FIc";
static struct option longopts[] =
{
  {"help", no_argument, 0, 'h'},
  {"getfeature", required_argument, 0, 'f'},
  {"getimg", required_argument, 0, 'b'},
  {"setstand", required_argument, 0, 'S'},
  {"getisofeature", no_argument, 0, 'F'},
  {"getisoimage", no_argument, 0, 'I'},
  {"convertimage", no_argument, 0, 'c'},
  {"testmatch", required_argument, 0, 'm'},
  {"testmatch2", required_argument, 0, 'n'},
  {"verifyall", no_argument, 0, 'V'},
  {"verifyuserid", required_argument, 0, 'v'},
  {"verifyfeature", no_argument, 0, 'Y'},
  {"delall", no_argument, 0, 'D'},
  {"delfinger", required_argument, 0, 'd'},
  {"getfinger", required_argument, 0, 'g'},
  {"getextra", required_argument, 0, 'G'},
  {"storefea", required_argument, 0, 's'},
  {"listall", no_argument, 0, 'L'},
  {"enroll", required_argument, 0, 'e'},
  {"getid", no_argument, 0, 'i'},
  {0, 0, 0, 0}
};

static void
usage (int status)
{
  if (status)
    fprintf (stderr, "Try ``command --help'' for more information.\n");
  else
    printf ("Usage: command [OPTION]\n"
	    "fingerprint io ctrl test.\n"
	    "\n"
	    "-h, --help                 display this help and exit\n"
	    "-f, --getfeature           get feature\n"
	    "-S, --setstand           	set stand\n"
	    "-F, --getisofeature        get isofeature\n"
	    "-I, --getisoimage          get isoimage\n"
	    "-c, --converformat         convert format\n"
	    "-m, --testmatch            test match\n"
	    "-n, --testmatch2           test match2\n"
	    "-V, --verifyall            verifyall\n"
	    "-v, --verifyuserid         verifyuserid\n"
	    "-Y, --verifyfeature        verifyfeature\n"
	    "-D, --delall            	delallfingers\n"
	    "-d, --delfinger           	delfinger\n"
	    "-g, --getfinger           	getuserfinger\n"
	    "-G, --getExtra           	getfingerextradata\n"
	    "-s, --storefea           	storefeature\n"
	    "-L, --listall            	listallfingers\n"
	    "-e, --enroll            	enroll\n"
	    "-i, --getid            	getid\n"
	    "\n"
	    );
}

static int savedata2file(unsigned char *pData, int dataLength, const char *pFileName)
{
	int fd = -1;
	int count = 0;
	
	printf("enter %s\n",__func__);

	
	fd = open(pFileName,O_RDWR|O_CREAT,0777);
	if(fd<0)
	{
	   fprintf(stderr,"Couldn't open '%s' (%s)\n", pFileName,strerror(errno));
	   return -1;
	}
	
	count = write(fd, pData, dataLength);
	if(count!=dataLength)
	{
	  fprintf(stderr,"%s write fail\n", pFileName);
	  close(fd);
	  return -1;
	}

	close(fd);

	return 0;
}

static int save2file(unsigned char *pfea, unsigned int nFeaLength, unsigned char *pimg, unsigned int nImgLength)
{
	int fd = -1;
	unsigned int count = 0;
	int len=0,i=0;

	printf("enter %s\n",__func__);

	fd = open("feature.data",O_RDWR|O_CREAT,0777);
	if(fd<0)
	{
	   fprintf(stderr,"Couldn't open '%s' (%s)\n", "feature.data",strerror(errno));
	   return -1;
	}
	
	count = write(fd, pfea, nFeaLength);
	if(count!=nFeaLength)
	{
	  fprintf(stderr,"feature.data write fail\n");
	  close(fd);
	  return -1;
	}

	close(fd);
	/*
        for(i=0;i<10;i++)
	{
		printf("%x\n",pimg[i+0x800]);
	} 
	*/
	fd = -1;
	fd = open("finger.bmp",O_RDWR|O_CREAT,0777);
        if(fd<0)
        {
           fprintf(stderr,"Couldn't open '%s' (%s)\n", "img.bmp",strerror(errno));
           return -1;
        }
	count = write(fd, pimg, nImgLength);
	if(count!=nImgLength)
                {
                     fprintf(stderr,"img.data write fail\n");
                     close(fd);
                     return -1;
                }
	close(fd);

	return 0;
}

static int save2fileext(unsigned char *pfea, unsigned int nFeaLength, char* feafilename, unsigned char *pimg, unsigned int nImgLength, char* imgfilename)
{
	int fd = -1;
	unsigned int count = 0;
	int len=0,i=0;

	printf("enter %s\n",__func__);

	if(pfea&&nFeaLength)
	{
	fd = open(feafilename,O_RDWR|O_CREAT,0777);
	if(fd<0)
	{
	   fprintf(stderr,"Couldn't open '%s' (%s)\n", feafilename,strerror(errno));
	   return -1;
	}
	
	count = write(fd, pfea, nFeaLength);
	if(count!=nFeaLength)
	{
	  fprintf(stderr,"%s write fail\n", feafilename);
	  close(fd);
	  return -1;
	}

	close(fd);
	}
	
	if(pimg&&nImgLength)
	{
	fd = -1;
	fd = open(imgfilename,O_RDWR|O_CREAT,0777);
        if(fd<0)
        {
           fprintf(stderr,"Couldn't open '%s' (%s)\n", imgfilename,strerror(errno));
           return -1;
        }
	count = write(fd, pimg, nImgLength);
	if(count!=nImgLength)
	{
		fprintf(stderr,"%s write fail\n", imgfilename);
		close(fd);
		return -1;
	}
	close(fd);
	}
	return 0;
}
//feature is 2k
static int getFeaFromFile(unsigned char *pfea, unsigned int *pFeaLength)
{
	int fd = -1;
        int count = 0;
	struct stat statbuf;

	printf("enter %s\n",__func__);
	if(stat("feature.data",&statbuf)==0)
		*pFeaLength = statbuf.st_size;
	else
        {
           fprintf(stderr,"Couldn't stat '%s' (%s)\n", "feature.data",strerror(errno));
           return -1;
        }

        fd = open("feature.data",O_RDONLY);
        if(fd<0)
        {
           fprintf(stderr,"Couldn't open '%s' (%s)\n", "feature.data",strerror(errno));
           return -1;
        }

        count = read(fd, pfea, statbuf.st_size);
        if(count!=statbuf.st_size)
        {
          fprintf(stderr,"feature.data read fail\n");
          close(fd);
          return -1;
        }

        close(fd);
	return 0;	
}

static int getFeaFromFileExt(unsigned char *pfea, unsigned int *pFeaLength)
{
	int fd = -1;
        int count = 0;
	struct stat statbuf;

	printf("enter %s\n",__func__);
	if(stat("feature2.data",&statbuf)==0)
		*pFeaLength = statbuf.st_size;
	else
        {
           fprintf(stderr,"Couldn't stat '%s' (%s)\n", "feature2.data",strerror(errno));
           return -1;
        }

        fd = open("feature2.data",O_RDONLY);
        if(fd<0)
        {
           fprintf(stderr,"Couldn't open '%s' (%s)\n", "feature2.data",strerror(errno));
           return -1;
        }

        count = read(fd, pfea, statbuf.st_size);
        if(count!=statbuf.st_size)
        {
          fprintf(stderr,"feature.data read fail\n");
          close(fd);
          return -1;
        }

        close(fd);
	return 0;	
}

static int finger_GetFea(unsigned char *pfea, unsigned int *pFeaLength, unsigned char *imgbuf, unsigned int *pImgLength)
{
     int i=0;
     int result=0xff;
     int realFeaLength = 0;
     int realImgLength = 0;
     int width, heigh;

     do{
		printf("ww get fea trying %d time\n",i);
		if(bOldInf)
			result=fp_get_fea(pfea, FEATURE_LEN, &realFeaLength, 120);
		else
     	 		result=fp_iso_get_fea(pfea, FEATURE_LEN, &realFeaLength, 120, nIsoType);
		printf("result=%d\n",result);
		i++;
	}while(i<100&&result);
    
     if(bOldInf)
	result = fp_getLastImage(imgbuf,IMG_TOTAL_LEN,&realImgLength,&width,&heigh);
     else	
     	result = fp_iso_getImage(imgbuf,IMG_TOTAL_LEN,&realImgLength,&width,&heigh, TYPE_NORMAL);
     printf("img res:%d,%d\n",width,heigh);
     printf("realFeaLength=%d\n",realFeaLength);
     printf("realImgLength=%d\n",realImgLength);

     *pFeaLength = realFeaLength;
     *pImgLength = realImgLength;

     return result;
}
/*
static int finger_GetISOFea(unsigned char *pfea, unsigned char *imgbuf)
{
     int i=0;
     int result=0xff;
     int realFeaLength = 0;
     int realImgLength = 0;
     int width, heigh;

     fp_setstand(1);
     
     do{
		printf("ww get isofea trying %d time\n",i);
     	 	result=fp_getisofeature(pfea, FEATURE_LEN, &realFeaLength, 120);
		printf("result=%d\n",result);
		i++;
	}while(i<100&&result);
    
     result = fp_getLastImage(imgbuf,IMG_TOTAL_LEN,&realImgLength,&width,&heigh);
     printf("img res:%d,%d\n",width,heigh);
     printf("realFeaLength=%d\n",realFeaLength); 
     return result;
}

static int finger_GetISOImage(unsigned char *pImage, int imageLength)
{
     int i=0;
     int result=0xff;
     int realImgLength = 0;

     fp_setstand(1);
     
     do{
		printf("ww get isoimage trying %d time\n",i);
     	 	result=fp_getisoimage(pImage, imageLength, &realImgLength, 120);
		printf("result=%d\n",result);
		i++;
	}while(i<100&&result);
    
     printf("realImgLength=%d\n",realImgLength);
     if(!result)
	savedata2file(pImage, realImgLength, "isoimg.data");	
     return result;
}
*/

static char * getcallbackmessage(int info)
{
	switch(info)
	{
		case FP_CB_INFO_NONE:
			return "FP_CB_INFO_NONE";
		case FP_CB_INFO_PUT_FINGER:
			return "FP_CB_INFO_PUT_FINGER";
		case FP_CB_INFO_REMOVE_FINGER:
			return "FP_CB_INFO_REMOVE_FINGER";
		case FP_CB_INFO_UNKNOWN:
			return "FP_CB_INFO_UNKNOWN";
		default:
			return "no info support";
	}
}

int fingerprintcallback(int info, unsigned char *pData, unsigned int nDataLen)
{
	printf("%s\n", getcallbackmessage(info));
	if(pData!=NULL&&nDataLen!=0)
		printf("%s\n", pData);
	return 0;
}

void *thread_func(void *arg)
{
	sp<ProcessState> proc(ProcessState::self());
	sp<IServiceManager> sm = defaultServiceManager();

	arg;

	fp_set_status_callback(fingerprintcallback);
	ProcessState::self()->startThreadPool();
	IPCThreadState::self()->joinThreadPool();

	return 0;
}

int main(int argc, char *argv[])
{
   int fd;
   int c, i;
   int command = 0, parameter = 0xffffffff;
   short result = 0;
   int nUserId;
   int reallenlength =0;
   pthread_t tid;

   unsigned int nFeaLength = 0;
   unsigned int nImgLength = 0;
   unsigned int nFileFeaLength = 0;
   unsigned int nFileFeaLength2 = 0;

   if(argc<2) 
   {
	printf("no command specified\n");
	exit(0);
   }

   printf("bin is:%s\n", argv[0]);
   if(strstr(argv[0], "oldinfingerprinttest"))
   {
	bOldInf = 1;
	printf("use old interface\n");
   }
   else
   {
	bOldInf = 0;
        printf("use new interface\n");
   }

   if(bOldInf)
	result = fp_open();
   else
   	result=fp_iso_open(); 

   if(result<0)
   {
	fprintf(stderr,"fingerprint open fail\n");
	return -1;
   }

   pthread_create(&tid,NULL,thread_func,NULL);

   do
    {
      c = getopt_long (argc, argv, optstring, longopts, 0);
      switch (c)
	{
	case EOF:
	  break;

	case 'h':
	  usage (0);
	  break;
        
	case 'f':
	{
	   nIsoType = atoi(optarg);
	   printf("nIsoType =%d\n", nIsoType);
	   result=finger_GetFea(fea_buf, &nFeaLength, img_buffer, &nImgLength);
	   if(!result)
	   {
		printf("get fea succ!\n");
		save2file(fea_buf, nFeaLength, img_buffer, nImgLength);
	   }
	   else
		fprintf(stderr,"get fea fail result=%d\n",result);
	}
           break;

	case 'b':
	{
	   int realImgLength = 0;
	   int width, heigh;

	   nIsoType = atoi(optarg);
	   printf("nIsoType =%d\n", nIsoType);
     	   if(bOldInf)
		result = fp_getLastImage(img_buffer,IMG_TOTAL_LEN,&realImgLength,&width,&heigh);
     	   else	
     		result = fp_iso_getImage(img_buffer,IMG_TOTAL_LEN,&realImgLength,&width,&heigh, nIsoType);

	   if(!result)
	   {
		printf("get img succ!\n");
		printf("img res:%d,%d\n",width,heigh);
		printf("realImgLength=%d\n",realImgLength);
		save2fileext(NULL, 0, NULL, img_buffer, realImgLength, (char *)"testfinger.bmp");
	   }
	   else
		fprintf(stderr,"get img fail result=%d\n",result);
	}
           break;
	

        case 'm':
	{
	   getFeaFromFile(fea_buf, &nFileFeaLength);
	   nIsoType = atoi(optarg);
           printf("nIsoType =%d\n", nIsoType);
           result=finger_GetFea(fea_buf_match, &nFeaLength, img_buffer, &nImgLength);
           if(!result)
           {
                short match_result=0;
                printf("get fea succ!\n");
		if(bOldInf)
			match_result = fp_match(fea_buf, nFileFeaLength, fea_buf_match, nFeaLength);
		else
			match_result = fp_iso_match(fea_buf, nFileFeaLength, fea_buf_match, nFeaLength);
		printf("match_result=%d\n",match_result);
		if(match_result<=0)
		  fprintf(stderr,"match error\n");
		else
		{
		  printf("match success!\n");
	          //save2file(fea_buf, nFeaLength, img_buffer, nImgLength);
		}
           }
	   else
		fprintf(stderr,"get fea fail result=%d\n",result);
	}
           break;

        case 'n':
	{
	   printf("match between files\n");
	   memset(fea_buf, 0x00, FEATURE_LEN);
	   memset(fea_buf_match, 0x00, FEATURE_LEN);
	   getFeaFromFile(fea_buf, &nFileFeaLength);
	   nIsoType = atoi(optarg);
           printf("nIsoType =%d\n", nIsoType);
	   getFeaFromFileExt(fea_buf_match, &nFileFeaLength2);
           {
                short match_result=0;
		if(bOldInf)
			match_result = fp_match(fea_buf, nFileFeaLength, fea_buf_match, nFileFeaLength2);
		else
                	match_result = fp_iso_match(fea_buf, nFileFeaLength, fea_buf_match, nFileFeaLength2);
		printf("match_result=%d\n",match_result);
		if(match_result<=0)
		  fprintf(stderr,"match error\n");
		else
		{
		  printf("match success!\n");
		}
           }
	}
           break;

	case 'e':
	{
	   char extra[64];
	   nUserId = atoi(optarg);
	   printf("nUserId=%d\n",nUserId);
	   sprintf(extra, "enrolltest%d",nUserId);
	   result = fp_iso_enroll(nUserId, 100000);//, (unsigned char *)extra, strlen(extra)+1);
	   printf("enroll result =%d\n", result);
	}
	   break;

	case 'V':
	{
	   nUserId = fp_iso_verifyall(100000);
	   printf("get userid=%d\n", nUserId);
	}
	   break;

	case 'D':
	{
	   result = fp_iso_delallfingers();
	   printf("del all fingers result=%d\n", result);
	}
	   break;

        case 'd':
	{
	   nUserId = atoi(optarg);
	   printf("nUserId=%d\n",nUserId);
	   result = fp_iso_delfinger(nUserId);
	   printf("del finger result =%d\n", result);
	}
           break;

        case 'g':
	{
	   nUserId = atoi(optarg);
	   printf("nUserId=%d\n",nUserId);
	   result = fp_iso_getuserfeature(nUserId, fea_buf, sizeof(fea_buf), &reallenlength, 0);
	   printf("get user finger result =%d, real len=%d\n", result, reallenlength);
	}
           break;

        case 'G':
	{
	   char extra[64];
	   nUserId = atoi(optarg);
	   printf("nUserId=%d\n",nUserId);
	   //result = fp_iso_getfingerextradata(nUserId, (unsigned char *)extra, sizeof(extra), &reallenlength);
	   printf("get user finger extra result =%d, real len=%d, extra data=%s\n", result, reallenlength, extra);
	}
           break;

        case 'v':
	{
	   nUserId = atoi(optarg);
	   printf("nUserId=%d\n",nUserId);
	   result = fp_iso_verifyagainstuserid(nUserId, 100000);
	   printf("verify userid result =%d\n", result);
	}
           break;

        case 's':
	{
	   char extra[64];
	   nUserId = atoi(optarg);
	   printf("nUserId=%d\n",nUserId);
	   sprintf(extra, "storetest%d",nUserId);
	   getFeaFromFile(fea_buf, &nFileFeaLength);
           result = fp_iso_storefeature(nUserId, fea_buf, nFileFeaLength/*sizeof(fea_buf)*/);//, (unsigned char *)extra, strlen(extra)+1);
	   printf("store feature result =%d\n", result);
	}
           break;

	case 'Y':
	{
	   getFeaFromFile(fea_buf, &nFileFeaLength);
	   result = fp_iso_verifyagainstfeature(fea_buf, nFileFeaLength, 100000);
	   printf("verify feature result =%d\n", result);
	}
	   break;
	
	case 'L':
	{
	   unsigned int userarray[100];
	   int fingercount;
	   result = fp_iso_listallfingers(userarray, 100, &fingercount);
	   printf("list all fingers result=%d\n", result);
	   printf("fingercount=%d\n",fingercount);
	   printf("fingers:\n");
	   for(i = 0; i< fingercount; i++)
		printf("%d,",userarray[i]);
	   printf("===========\n");
	}
	   break;
        
	case 'S':
	{
	   int nType = atoi(optarg);
	   printf("nType=%d\n",nType);
	   //result = fp_setstand(nType);
	   printf("set stand result =%d\n", result);
	}
	   break;

	case 'F':
	{
	   //result=finger_GetISOFea(fea_buf,img_buffer);
	   if(!result)
	   {
		printf("get isofea succ!\n");
		save2file(fea_buf, nFeaLength, img_buffer, nImgLength);
	   }
	   else
		fprintf(stderr,"get isofea fail result=%d\n",result);
	}
           break;

	case 'I':
	{
	   //result=finger_GetISOImage(isoimage,sizeof(isoimage));
	   if(!result)
	   {
		printf("get isoimage succ!\n");
	   }
	   else
		fprintf(stderr,"get isoimage fail result=%d\n",result);
	}
           break;
        
	case 'c':
	{
	   int nRealFeaLength = 0;
	   getFeaFromFile(fea_buf, &nFileFeaLength);
	   result=fp_iso_convertformat(fea_buf, nFileFeaLength, TYPE_NORMAL, fea_buf_match, FEATURE_LEN, &nRealFeaLength, TYPE_ISO_2005);
           if(!result)
           {
		savedata2file(fea_buf_match, nRealFeaLength, "convert2isofeature.data");
		printf("convertformat succ\n");
           }
	   else
		fprintf(stderr,"convertformat fail result=%d\n",result);
	}
           break;

	case 'i':
	{	
	   const char* iddes[]={"crossmatch", "tuzheng", "tuzhengbig", "aratek", "next"};
	   int id;
	   if(bOldInf)
		id = fp_getid();
	   else
	   	id = fp_iso_getid();
	   if(-1 != id)
		printf("fp id:%s\n",iddes[id]);
	    else
		printf("get id failed\n");
	}
	   break;

	default:
	  usage (1);
	  break;
	}
    }
  while (c != EOF);

  if(bOldInf)
	result = fp_close();
  else
  	result = fp_iso_close();

  printf("exit %s.....\n", __func__);
  if(result<0)
	fprintf(stderr,"fingerprint close fail\n");

end:
   return 0;   
}

