/*
 * pinpad_interface.h
 *
 *  Created on: 2012-8-2
 *      Author: yaomaobiao
 */

#ifndef PINPAD_INTERFACE_H_
#define PINPAD_INTERFACE_H_

#define KEY_TYPE_DUKPT        0
#define KEY_TYPE_TDUKPT        1
#define KEY_TYPE_MASTER        2
#define KEY_TYPE_PUBLIC        3
#define KEY_TYPE_FIX        4

#ifndef KEY_TYPE_TDUKPT2009
#define KEY_TYPE_TDUKPT2009    5
#endif

#include <hal_pinpad_interface.h>

#ifdef __cplusplus
extern "C"
{
#endif

#ifndef PINPAD_TYPE_NL_PP60
#define PINPAD_TYPE_NL_PP60		0
#endif

#ifndef PINPAD_TYPE_NL_GP710
#define PINPAD_TYPE_NL_GP710	1
#endif

/*
 *int nCount : the count of characters that are supposed to be showed in the user own soft pinpad 
 *int nExtra : extra message.
 */
typedef void (*KEYEVENT_NOTIFIER)(int nCount, int nExtra);
/*
 * open the device
 * return value : 0 : success
 * 				  < 0 : error code
 */
typedef int (*pinpad_open)();
/*
 * close the device
 * return value : 0 : success
 * 				  < 0 : error code
 */
typedef int (*pinpad_close)();
/*
 * show text in the first line
 * param[in] : int nLineIndex : line index, from top to down
 * param[in] : char* strText : encoded string
 * param[in] : int nLength : length of string
 * param[in] : int nFlagSound : 0 : no voice prompt, 1 : voice prompt
 * return value : < 0 : error code, maybe, your display string is too long!
 * 				  >= 0 : success
 *
 */
typedef int (*pinpad_show_text)(int nLineIndex, char *strText, int nLength,
                                int nFlagSound);
/*
 * select master key and user key
 * @param[in] : int nKeyType : 0 : dukpt, 1: Tdukpt, 2 : master key, 3 : public key, 4 : fix key, 5 : Tdukpt2009
 * @param[in] : int nMasterKeyID : master key ID , [0x00, ..., 0x09], make sense only when nKeyType is master-session pair,
 * @param[in] : int nUserkeyID : user key ID, [0x00, 0x01], 		  make sense only when nKeyType is master-session pair,
 * @param[in] : int nAlgorith : 1 : 3DES
 * 							   0 : DES
 * return value : < 0 : error code
 * 				  >= 0 : success
 */
typedef int (*pinpad_select_key)(int nKeyType, int nMasterKeyID, int nUserKeyID,
                                 int nAlgorith);

/*
 * encrypt string using user key
 * @param[in] : unsigned char* pPlainText : plain text
 * @param[in] : int nTextLength : length of plain text
 * @param[out] : unsigned char* pCipherTextBuffer : buffer for saving cipher text
 * @param[in] : int CipherTextBufferLength : length of cipher text buffer
 * return value : < 0 : error code
 * 				  >= 0 : success, length of cipher text length
 */
typedef int (*pinpad_encrypt_string)(unsigned char *pPlainText, int nTextLength,
                                     unsigned char *pCipherTextBuffer, int nCipherTextBufferLength);

/*
 * encrypt string using user key
 * @param[in] : unsigned char* pPlainText : plain text
 * @param[in] : int nTextLength : length of plain text
 * @param[out] : unsigned char* pCipherTextBuffer : buffer for saving cipher text
 * @param[in] : int CipherTextBufferLength : length of cipher text buffer
 * @param[in] : unsigned int nMode : 
 *					PINPAD_ENCRYPT_STRING_MODE_EBC  0
 *					PINPAD_ENCRYPT_STRING_MODE_CBC	1
 *					PINPAD_ENCRYPT_STRING_MODE_CFB	2
 *					PINPAD_ENCRYPT_STRING_MODE_OFB	3
 * @param[in] : unsigned char* PIV : initial vector, only for CBC, CFB, OFB mode
 * @param[in] : unsigned int nIVLen : length of IV, must be equal to block length according to the algorithm
 * return value : < 0 : error code
 * 				  >= 0 : success, length of cipher text length
 */
typedef int (*pinpad_encrypt_string_with_mode)(unsigned char *pPlainText, int nTextLength,
                                               unsigned char *pCipherTextBuffer,
                                               int nCipherTextBufferLength, unsigned int nMode,
                                               unsigned char *pIV, unsigned int nIVLen);

/*
 * calculate pin block
 * @param[in] : unsigned char* pASCIICardNumber : card number in ASCII format
 * @param[in] : int nCardNumberLength : length of card number
 * @param[out] : unsigned char* pPinBlockBuffer : buffer for saving pin block
 * @param[in] : int nPinBlockBufferLength : buffer length of pin block
 * @param[in] : int nTimeout_MS : timeout waiting for user input in milliseconds, if it is less than zero, then wait forever
 * param[in]   : int nFlagSound : 0 : no voice prompt, 1 : voice prompt
 * return value : < 0 : error code
 * 			      >= 0 : success, length of pin block
 */
typedef int (*pinpad_calculate_pin_block)(unsigned char *pASCIICardNumber,
                                          int nCardNumberLength, unsigned char *pPinBlockBuffer,
                                          int nPinBlockBufferLength, int nTimeout_MS,
                                          int nFlagSound);

/*
 * calculate the MAC using current user key
 * @param[in] : unsigned char* pData : data
 * @param[in] : int nDataLength : data length
 * @param[in] : int nMACFlag : 0: X99, 1 : ECB
 * @param[out] : unsigned char* pMACOut : MAC data buffer
 * @param[in] : int nMACOutBufferLength : length of MAC data buffer
 * return value : < 0 : error code
 * 				  >= 0 : success
 *
 */
typedef int (*pinpad_calculate_mac)(unsigned char *pData, int nDataLength,
                                    int nMACFlag, unsigned char *pMACOutBuffer,
                                    int nMACOutBufferLength);

/*
 * update the user key
 * @param[in] : int nMasterKeyID : master key id
 * @param[in] : int nUserKeyID : user key id
 * @param[in] : unsigned char* pCipherNewUserkey : new user key in cipher text
 * @param[in] : int nCipherNewUserKeyLength : length of new user key in cipher text
 * return value : < 0 : error code
 * 				  >= 0 : success
 */
typedef int (*pinpad_update_user_key)(int nMasterKeyID, int nUserKeyID,
                                      unsigned char *pCipherNewUserKey,
                                      int nCipherNewUserKeyLength);

/*
 * set the max length of pin
 * @param[in] : int nLength : length >= 0 && length <= 0x0C
 * @param[in] : int nFlag : 1, max length
 * 							0, min length
 * return value : < 0 : error code
 * 				  >= 0 : success
 */
typedef int (*pinpad_set_pin_length)(int nLength, int nFlag);

/*
 * update the master key
 * @param[in] : int nMasterKeyID : master key ID
 * @param[in] : unsigned char* pOldKey, old key
 * @param[in] : int nOldKeyLength : length of old key, 8 or 16
 * @param[in] : unsigned char* pNewLey : new key
 * @param[in] : int nNewLeyLength : length of new key, 8 or 16
 * return value : < 0 : error code
 * 				  >= 0 : success
 */
typedef int (*pinpad_update_master_key)(int nMasterKeyID, unsigned char *pOldKey, int nOldKeyLength,
                                        unsigned char *pNewKey, int nNewKeyLength);


typedef int (*pinpad_update_user_key_with_tr31format)(int nMasterKeyID, int nUserKeyID,
                                                      unsigned char *pTr31Msg, int nTr31MsgLen);
/*
 * get serial number
 * @param[out] : unsigned char* pData : serial number buffer
 * @param[in]  : unsigned int nBufferLength : length of serial number buffer
 * return value : < 0 : error code
 * 				  >= 0 : success, length of serial number
 */
typedef int (*pinpad_get_serial_number)(unsigned char *pData, unsigned int nBufferLength);

/*
 * enable or disable hardware 
 * @param[in] : unsigned int nEable : 1 : enable, 0 : disable
 * return value : >= 0 : success
 *                < 0 : error code
 */
typedef int (*pinpad_setEnable)(unsigned int nEnable);

/*
 * set the pinpad callback
 */
typedef int (*pinpad_set_pinblock_callback)(KEYEVENT_NOTIFIER pNotifier);


typedef int (*pinpad_update_user_key_with_check)(int nMasterKeyID, int nUserKeyID,
                                                 unsigned char *pCipherNewUserKey,
                                                 int nCipherNewUserKeyLength, int nKeyUsge,
                                                 unsigned char *pCheckValue, int nCheckValueLen);
typedef int (*pinpad_update_cipher_master_key)(int nMasterKeyID, unsigned char *pCipherMasterKey,
                                               int nCipherMasterKeyLen, unsigned char *pCheckValue,
                                               int nCheckValueLen);
typedef int (*pinpad_update_user_key_with_check_E)(int nMasterKeyID, int nUserKeyID,
                                                   unsigned char *pCipherNewUserKey,
                                                   int nCipherNewUserKeyLength, int nKeyUsge,
                                                   unsigned char *pCheckValue, int nCheckValueLen,
                                                   int algoCheckValue);
typedef int (*pinpad_update_cipher_master_key_E)(int nMasterKeyID, unsigned char *pCipherMasterKey,
                                                 int nCipherMasterKeyLen,
                                                 unsigned char *pCheckValue, int nCheckValueLen,
                                                 int algoCheckValue);
/*
 * New function, Added by SUN, 2016/08/02
 * for customer from Israel
 * @param[in] : unsigned int nFlag : 1 : allow bypass pin, No pin input is allowed, user can input Enter directly.
 * 									 0 : forbid bypass pin, No pin input is not allowed.
 * return value : < 0 : error code
 * 				  >= 0 : success
 */
typedef int (*pinpad_set_flag_allow_bypass_pin)(unsigned int nFlag);


typedef int (*pinpad_get_hwserialno)(unsigned char *pData, unsigned int nBufferLength);

typedef int (*pinpad_get_mac_for_snk)(unsigned char *pData, unsigned int nBufferLength,
                                      unsigned char *pRandomData, unsigned int nRandomDataLength,
                                      unsigned char *pMAB, unsigned int nMABLength);

/*
 * update the master key after whit the specified master key decryption.
 * @param[in] : int nDecryptMasterKeyID : decryption master key ID 
 * @param[in] : int nMasterKeyID : update master key ID
 * @param[in] : unsigned char* pCipherMK : cipher master key to be saved 
 * @param[in] : int nNewLeyLength : length cipher master key
 * @param[in] : pCheckValue : check value, you can set it to NULL
 * @param[in] : unsigned int nCheckValueLen, if pCheckValue is NULL, you should set it to zero
 * @param[in] : unsigned int algoCheckValue
 * return value : < 0 : error code
 * 				  >= 0 : success
 */
typedef int (*pinpad_update_specified_master_key)(unsigned int nDecryptMasterKeyID,
                                                  unsigned int nMasterKeyID,
                                                  unsigned char *pCipherMK,
                                                  unsigned int nCipherMKLen,
                                                  unsigned char *pCheckValue,
                                                  unsigned int nCheckValueLen,
                                                  unsigned int algoCheckValue);

/*
 * select the pinblock format
 * @param[in] : unsigned int nPinblockFormat : 0~5, ISO0/ISO1/ISO2/ISO3/ISO4/SM4
 * return value : < 0 : error code
 *                >= 0 : success.
 * Attention : in HAL layer, we only support ISO0, ISO2, ISO4, SM4. 
 *             Case 0 : ISO2 is only for the function of IC card offline pin, pinblock is saved in the hardware security module. 
 *                      HAL layer can not get it because it is not encrypted with a key.
 *             Case 1 : If you inject key with a specified algorithm and this algorithm is mismatched with the pinblock format,
 *                      hardware security module will return an error code. If they are matched, you are lucky!
 *             Case 2 : If you inject key without a specified algorithm, hardware security will calculate pinblock according to your choice.
 *             Case 3 : If you did not select pinblock format, hardware security module will select pinblock format 
 *                      according to the algorithm you specified in the api hal_pinpad_select_key
 *
 */
typedef int (*pinpad_select_pinblock_format)(unsigned int nPinblockFormat);


/* added by maobiao.yao@wizarpos.com, 2019/02/22 */
/* calcualte check value using specified master key, only allowed when the TK is existence.
 * @param[in] : unsigned int nMKID : master key ID;
 * @param[in] : unsigned int nCheckValueType : now only support the default format, must be specified with 0
 * @param[out] : unsigned char* pCheckValueBuf : buffer for the final result
 * @param[in] : unsigned int nCheckValueBufLen : length of buffer, must be greater than or equal to 4
 * return value : < 0 : error code
 *                > 0 : length of check value
 *                never return 0;
 * */
typedef int (*pinpad_get_mk_checkvalue)(unsigned int nMKID, unsigned int nAlgo,
                                        unsigned char *pCheckValueBuf,
                                        unsigned int nCheckValueBufLen);

/* added by maobiao.yao@wizarpos.com, 2019/02/22 */
/*
 * calculate check value using specified session key. make sure that you have updated the session key in this slot
 * @param[in] : unsigned int nMKID : master key ID;
 * @param[in] : unsigned int nSKID : session key ID;
 * @param[in] : unsigned int nCheckValueType : now only support the default format, must be specified with 0
 * @param[out] : unsigned char* pCheckValueBuf : buffer for the final result
 * @param[in] : unsigned int nCheckValueBufLen : length of buffer, must be greater than or equal to 4
 * @param[in] : unsigned int nCheckValueBufLen : length of buffer, must be greater than or equal to 4
 * return value : < 0 : error code
 *                > 0 : length of check value
 *                never return 0;
 */
typedef int (*pinpad_get_sk_checkvalue)(unsigned int nMKID, unsigned int nSKID, unsigned int nAlgo,
                                        unsigned char *pCheckValueBuf,
                                        unsigned int nCheckValueBufLen);

/*
 * Verify the MAC
 * @param[in] : unsigned char* pData : data
 * @param[in] : unsigned int nDataLen : data length
 * @param[in] : unsigned int nMACFlag : 0: X99, 1 : ECB, 2 : SE919, 3 : Union pay ECB
 * @param[in] : unsigned char* pMacData : mac data
 * @param[in] : unsigned int nMacDataLen : mac data length, >=4 && <=8
 * @param[in] : unsigned int nDirection : must be 0, reserved for future
 */
typedef int (*pinpad_verify_response_mac)(unsigned char *pData, unsigned int nDataLen,
                                          unsigned int nMacFlag, unsigned char *pMacData,
                                          unsigned int nMacDataLen, unsigned int nDirection);

/**
 * Set PINPAD GUI configuration. When setting flag 1 and 2, the setting will keep forever.
 *
 * @param nFlag       1 to set asterisk, 2 to set title bar
 * @param pConfigData when flag is 1, 0x00: Left, 0x01: Center, 0x02: Right; when flag is 2, the title content.
 * @return value : < 0 : error code
 * >= 0 : success
 */
typedef int (*pinpad_set_gui_configuration)(unsigned int nMacFlag, unsigned char *pMacData,
                                            unsigned int nMacDataLen);

/* get the status of MK with the specified ID  */
/*
 * @param       : nMKID : master key ID
 * return value : < 0 : error code
 *                0 : it does not exist
 *                1 : it exists
 * */
typedef int (*pinpad_get_mk_status)(unsigned int nMKID);

/* get the status of SK with the specified MKID and SKID */
/*
 * @param: nMKID  : master key ID
 * return value  : < 0 : error code
 *                 0 : it does not exist
 *                 1 : it exists
 * */
typedef int (*pinpad_get_sk_status)(unsigned int nMKID, unsigned int nSKID);

/*
 * get the status of DUKPT Key
 * @param       : int nDukptID : ID of dukpt
 * @param       : byte[] pKSNBuf : buffer for saving KSN of this dukpt key
 * return vlaue : < 0 : error code
 *                0 : it does not exist
 *                > 0 : KSN data length
 */

typedef int (*pinpad_get_dukpt_status)(unsigned int nDukptID, unsigned char *pKSNBuf,
                                       unsigned int nKSNBufLen);

/*
 * create the SK in cipher text
 * @param[in] : unsigned int nMasterKeyID : master key ID, [0-49]
 * @param[out]: unsigned char* pCipherSkBuf : buffer for SK
 * @param[in] : unsigned int nCipherSkBufLen : buffer length.
 *                                      length of buffer must be greater than the length of expected SK
 *                                      Please pay attention, for AES key with the length of 24, the cipher SK length is 32
 * @param[in] : unsigned int nExpectedSKLen : length of SK
 *                                      Algorithm TDEA : 8,16,24
 *                                      Algorithm SM4 : 16
 *                                      Algorithm AES : 16, 24, 32
 * @param[in] : unsigned int nAlgorithm : 0 : reserved
 *                                        1 : TDEA
 *                                        2 : SM4
 *                                        3 : AES
 * @param[in] : unsigned int nParityCheck : 0 : no parity check
 *                                          1 : odd
 *                                          2 : even
 * return value :
 *              >= 0 : data length in the buffer
 *              < 0 : error code
 *
 * */
typedef int (*pinpad_create_sk)(unsigned int nMasterKeyID, unsigned char *pCipherSkBuf,
                                unsigned int nCipherSkBufLen, unsigned int nExpectedSkLen,
                                unsigned int nAlgorithm, unsigned int nParityCheck);

#ifdef __cplusplus
}
#endif

#endif /* PINPAD_INTERFACE_H_ */
